#include "common.h"

#define MAX_BUTTONS 100

typedef struct
{
  int exitcode;
  char *label;
} tbuttons;

int choosen;

/*************************************************************************/
void timeout_expired(int sig) {
    exit(0);
}

/*************************************************************************/
void button_clicked(GtkWidget *Dummy, int number)
{
  choosen = number;
  gtk_main_quit();
}

/*************************************************************************/
int gmessage(const char *message, tbuttons *buttons,
             int nb_buttons, int _default, int print, int position)
{
  GtkWidget *window;
  GtkWidget *vbox, *hbox, *scrolled_window;
  GtkTextView *text;
  GtkTextBuffer *textbuf;
  int i;

  choosen = _default;
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "gmessage");
  gtk_window_set_position(GTK_WINDOW(window), position);

  gtk_signal_connect(GTK_OBJECT(window), "destroy",
                     (GtkSignalFunc)gtk_main_quit, NULL);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     (GtkSignalFunc)gtk_main_quit, NULL);
    
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_widget_show(vbox);

  PACK11(vbox, scrolled_window = gtk_scrolled_window_new(NULL, NULL) );
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_widget_set_usize(GTK_WIDGET(scrolled_window), 550, 130);
    
  textbuf = gtk_text_buffer_new(NULL);
  text = GTK_TEXT_VIEW(gtk_text_view_new_with_buffer(textbuf));
  gtk_text_view_set_editable(text, 0);
  gtk_text_view_set_cursor_visible(text, 0);
  gtk_text_buffer_set_text(textbuf, message, -1);

  gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(text));
  gtk_widget_show(GTK_WIDGET(text));

  PACK00(vbox, hbox = gtk_hbox_new(FALSE, 0));

  for (i = 0; i < nb_buttons; i++)
  {
    GtkWidget *button;
    
    PACK11(hbox, button = gtk_button_new_with_label(buttons[i].label));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(button_clicked), (gpointer) i);
    if (i == _default)
      gtk_widget_grab_focus(button);
  }

  gtk_widget_show(window);
  gtk_main();
  alarm(0);
  return choosen >= 0 ? buttons[choosen].exitcode : 1;
}

/*************************************************************************/
int main(int argc, char **argv)
{
  char *message = NULL;
  tbuttons buttons[MAX_BUTTONS];
  int nb_buttons = 0, print = 0;
  char *cdefault = NULL;
  int _default = -1;
  int i;
  int position = GTK_WIN_POS_NONE;
  struct sigaction s;
  char *endptr;
  long timeout_val;

  initlocale("gtk+mdk");

  for (argc--, argv++; argc; argc--, argv++)
  {
    if (streq(*argv, "-file"))
    {
      FILE *f;
      struct stat s;

      argc--; argv++;
      if (!argc)
        die(_("missing the filename after -file\n"));
      if ((f = fopen(*argv, "r")) == NULL)
        die(_("cannot open file %s\n"), *argv);
      if (fstat(fileno(f), &s))
        die(_("cannot stat file %s\n"), *argv);
      if ((message = malloc(s.st_size)) == NULL)
        die(_("malloc failed\n"));
      if (fread(message, 1, s.st_size, f) != s.st_size)
        die(_("error reading\n"));
      if (fclose(f))
        die(_("fclose failed\n"));
    } else if (streq(*argv, "-buttons"))
    {
      char *p, *q, *r;
      int i;

      argc--; argv++;
      if (!argc)
        die(_("missing the comma-separated list of label:exitcode\n"));
      for (i = 0, p = q = strdup(*argv); i < MAX_BUTTONS && q; p = q + 1, i++)
      {
        int j,k;
	char *label;
        if ((q = strchr(p, ','))) {
	  while (q > p && *(q-1) == '\\')
	    q = strchr(q + 1, ',');
	  if (q)
	    *q = 0;
	}

        if ((r = strchr(p, ':')))
        {
          *r = 0;
          buttons[i].exitcode = atoi(r + 1);
        } else buttons[i].exitcode = 100 + 1 + i;

        /* now copy char per char to s|\,|,| */
	buttons[i].label = strdup(p);
	bzero(buttons[i].label, strlen(p));
        for (j=0,k=0; j<strlen(p); j++,k++) {
	  buttons[i].label[k] = p[j];
          if (p[j] == '\\' && p[j+1] == ',')
	    k--;
	}

      }
      nb_buttons = i;
    } else if (streq(*argv, "-default"))
    {
      argc--; argv++;
      if (!argc)
        die(_("missing the label of the default button\n"));
      cdefault = *argv;
    } else if (streq(*argv, "-print"))
    {
      print = 1;
    } else if (streq(*argv, "-center"))
    {
      position = GTK_WIN_POS_CENTER;
    } else if (streq(*argv, "-nearmouse"))
    {
      position = GTK_WIN_POS_MOUSE;
    } else if (streq(*argv, "-timeout"))
    {
      argc--; argv++;
      if (!argc)
        die(_("missing the number of seconds before timeout\n"));
      s.sa_handler=timeout_expired;
      sigemptyset(&s.sa_mask);
      s.sa_flags=0;
      sigaction(SIGALRM, &s, NULL);
      timeout_val = strtol(*argv, &endptr, 10);
      if (errno == ERANGE)
        die(_("timeout value out of range\n"))
      else if (timeout_val > UINT_MAX)
        die(_("timeout value too large\n"))
      else if (timeout_val < 0)
        die(_("timeout value too small\n"))
      else if (*argv==endptr)
        die(_("timeout needs numeric value\n"))
      alarm(timeout_val);
    } else break;
  }

  if (((message != 0) ^ argc) == 0)
    die(_("usage: gmessage [-options] [message ...]\n\
\n\
where options include:\n\
    -file filename              file to read message from, \"-\" for stdin\n\
    -buttons string             comma-separated list of label:exitcode\n\
    -default button             button to activate if Return is pressed\n\
    -print                      print the button label when selected\n\
    -center                     pop up at center of screen\n\
    -nearmouse                  pop up near the mouse cursor\n\
    -timeout secs               exit with status 0 after \"secs\" seconds\n\
"));

  if (argc)
  {
    message = g_strdup("");
    for (; argc; argc--, argv++)
    {
      gchar *p = g_strconcat(message, *argv, " ", NULL);
      g_free(message);
      message = p;
    }
  }

  if (nb_buttons == 0)
  {
    nb_buttons = 1;
    buttons[0].label = _(" OK ");
    buttons[0].exitcode = 0;
  }

  if (cdefault)
    for (i = 0; i < nb_buttons; i++)
      if (streq(cdefault, buttons[i].label))
      { _default = i; break; }

  gtk_init(NULL, NULL);
  exit(gmessage(message, buttons, nb_buttons, _default, print, position));
}
